# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see
# <http://www.gnu.org/licenses/>.

""" AMQP RPC type event consumer related classes and functions. """

from __future__ import absolute_import
from .exchange import publish_message
from .exceptions import MessageNotDelivered, MessageDeliveryTimeout
from .queue import Queue
import logging
import time
import uuid


class Rpc(Queue):
	"""
	Attempts to simplify RPC style calls using AMQP exchanges and queues. This works two-fold:
	* This class will create an Exchange for sending RPC messages.
	* This class will also create a Queue for handling responses to the RPC messages.
	Additionally, this class can also create a 'normal' Queue, to avoid having to create a separate instance.
	All of the above is created using a single AMQP channel.
	"""
	def __init__(self, host, credentials, identifier=None, prefetch_count=1, exchange=None, auto_delete=True, queue=None, binds=None, confirm_delivery=False):
		"""
		Initialize AMQP connection.

		Parameters
		----------
		host: tuple
			Must contain hostname and port to use for connection
		credentials: tuple
			Must contain username and password for this connection
		identifier: string
			Identifier for this RPC Queue. This parameter determines what the incoming queue will be called.
			If left as None, an identifier will be generated.
		prefetch_count: int
			Set the prefetch_count of all queues defined by this class.
		exchange: string
			If set, this RPC queue will also be bound to the given exchange using the identifier as routing key. If not,
			we will only be implicitly bound to the default exchange
		auto_delete: boolean
			Set to True to automatically delete the created RPC queue.
		queue: dict
			Create a separate queue based on the given parameters. This is a general purpose AMQP queue, and can be
			provided a separate callback using consume().
			Must contain at least the following keys:
				queue: string - what queue to use
				passive: boolean - should we use an existing queue, to try to declare our own
			Options below are optional when passive = True
				durable: boolean - should the queue be durable
				auto_delete: boolean - should we auto delete the queue when we close the connection
		binds: list of dicts
			A list of dicts  with the following keys:
				queue: string - name of the queue to bind
				exchange: string - name of the exchange to bind
				routing_key: string - routing key to use for this bind
		confirm_delivery: boolean
			If True, basic.Confirm will be set on the current channel.
		"""
		self.logger = logging.getLogger(__name__)

		self.rpc_queue_name = identifier
		if not self.rpc_queue_name:
			self.rpc_queue_name = "rpc.{0}".format(int(time.time()))

		rpc_queue = {
			"queue": self.rpc_queue_name,
			"passive": False,
			"durable": False,
			"auto_delete": auto_delete
		}
		if exchange:
			binds.append({"queue": self.rpc_queue_name, "exchange": exchange, "routing_key": self.rpc_queue_name})

		super(Rpc, self).__init__(host, credentials, rpc_queue, None)

		if queue:
			self.queue_name = queue['queue']
			self.logger.info("Declaring general purpose queue {0}".format(self.queue_name))
			self.channel.queue_declare(**queue)
		else:
			del(self.queue_name)

		if binds:
			self._perform_binds(binds)

		self.channel.basic_qos(prefetch_count=prefetch_count)

		if confirm_delivery:
			self.channel.confirm_delivery()

		self.responses = {}

	def consume(self, consumer_callback=None, exclusive=False):
		"""
		Initialize consuming of messages from an AMQP RPC queue. Messages will be consumed after start_consuming() is called.

		An internal callback will be used to handle incoming RPC responses. Only responses that have been registered with register_response()
		will be kept internally, all other responses will be dropped silently. Responses can be accessed by using get_response().

		The internal callback will assume that the incoming RPC responses will have a correlation_id property set in the headers.

		Additionally, if a general purpose queue was created on construction, the parameters to this function can be used to declare a callback
		and options for that queue. A ValueError is raised when trying to set a general purpose callback, but no queue was declared during
		construction.

		In contrast to the Queue class, the recover parameter is missing from this implementation of consume(). We will always try to requeue
		old messages.

		Parameters
		----------
		consumer_callback: callback
			Function to call when a message is consumed. The callback function will be called on each delivery,
			and will receive three parameters:
				* channel
				* method_frame
				* header_frame
				* body
		exclusive: boolean
			Is this consumer supposed to be the exclusive consumer of the given queue?
		"""
		if not hasattr(self, "queue_name") and consumer_callback:
			raise ValueError("Trying to set a callback, while no general purpose queue was declared.")

		self.rpc_consumer_tag = self.channel.basic_consume(consumer_callback=self._rpc_response_callback, queue=self.rpc_queue_name, exclusive=False)

		if consumer_callback:
			super(Rpc, self).consume(consumer_callback, exclusive, True)

	def _rpc_response_callback(self, channel, method_frame, header_frame, body):
		"""
		Internal callback used by consume()

		Parameters
		----------
		channel: object
			Channel from which the callback originated
		method_frame: dict
			Information about the message
		header_frame: dict
			Headers of the message
		body: string
			Body of the message
		"""
		self.logger.debug("Received RPC response with correlation_id: {0}".format(header_frame.correlation_id))
		if header_frame.correlation_id in self.responses:
			self.responses[header_frame.correlation_id] = {
				"method_frame": method_frame,
				"header_frame": header_frame,
				"body": body
			}
		channel.basic_ack(method_frame.delivery_tag)

	def register_response(self, correlation_id=None):
		"""
		Register the receiving of a RPC response. Will return the given correlation_id after registering, or if correlation_id is None, will
		generate a correlation_id and return it after registering. If the given correlation_id has already been used, an KeyError will be
		raised.

		UUID version 1 will be used when generating correlation_ids. Depending on the underlying system and implementation, this will guarantee
		that generated values are unique between workers. At least CPython guarantees this behaviour.

		Parameters
		----------
		correlation_id: string
			Identifier under which to expect a RPC callback. If None, a correlation_id will be generated.

		"""
		if not correlation_id:
			correlation_id = str(uuid.uuid1())

		if correlation_id in self.responses:
			raise KeyError("Correlation_id {0} was already registered, and therefor not unique.".format(correlation_id))

		self.responses[correlation_id] = None
		return correlation_id

	def retrieve_available_responses(self):
		"""
		Retrieve a list of all available responses. Will return a list of correlation_ids.
		"""
		return [k for (k, v) in self.responses.iteritems() if v]

	def retrieve_response(self, correlation_id):
		"""
		Retrieve a registered RPC response. If the correlation_id was not registered, an KeyError will the raised. If not value has been
		received yet, None will be returned. After retrieving the response, the value will be unset internally.

		The returned value will include the entire RabbitMQ message, consisting of a dict with the following keys:

			method_frame: dict
				Information about the message
			header_frame: dict
				Headers of the message
			body: string
				Body of the message

		Parameters
		----------
		correlation_id: string
			Identifier to retrieve the RPC response for
		"""
		if correlation_id not in self.responses:
			raise KeyError("Given RPC response correlation_id was not registered.")
		if not self.responses[correlation_id]:
			return None

		response = self.responses[correlation_id]
		del(self.responses[correlation_id])
		return response

	def request_response(self, exchange, routing_key, message, properties=None, correlation_id=None, timeout=6):
		"""
		This function wraps publish, and sets the properties necessary to allow end-to-end communication using the Rpc paradigm.

		This function assumes that the named exchange and routing_key combination will result in a AMQP queue to pickup the request, and
		reply using another AMQP message. To achieve this, the following properties are set, along with any custom properties:
		* correlation_id: a correlation_id is generated using register_response. It is assumed that the responding service will also provide
			the same id in the response.
		* reply_to: this is set to the internal RPC queue name, so we can pickup responses.

		The mandatory bit will be set on the message as a mechanism to detect if the message was delivered to a queue.
		This is to avoid needlessly waiting on a reply, when the message wasn't delivered in the first place.

		Parameters
		----------
		exchange: string
			Exchange to publish to.
		routing_key: string
			Routing key to use for this message.
		message: string
			Message to publish.
		properties: dict
			Properties to set on message. This parameter is optional, but if set, at least the following options must be set:
				content_type: string - what content_type to specify, default is 'text/plain'.
				delivery_mode: int - what delivery_mode to use. By default message are not persistent, but this can be
					set by specifying PERSISTENT_MESSAGE .
		correlation_id: string
			Custom correlation_id. This identifier is subject to the same semantics and logic as register_response().
		timeout: int
			How many seconds to wait for a reply. If no reply is received, an MessageDeliveryTimeout is raised. Set to
			False to wait forever.
		"""
		if not properties:
			properties = {}
		properties['correlation_id'] = self.register_response(correlation_id)
		properties['reply_to'] = self.rpc_queue_name

		if not self.publish(exchange, routing_key, message, properties, mandatory=True):
			self.retrieve_response(properties['correlation_id'])
			raise MessageNotDelivered("Message was not delivered to a queue")

		start = int(time.time())

		## Newer versions of pika (>v0.10) don't have a force_data_events any more
		if hasattr(self.channel, "force_data_events"):
			self.channel.force_data_events(True)

		while properties['correlation_id'] not in self.retrieve_available_responses():
			self.connection.process_data_events()
			if timeout and (int(time.time()) - start) > timeout:
				self.retrieve_response(properties['correlation_id'])
				raise MessageDeliveryTimeout("No response received from RPC server within specified period")

		return self.retrieve_response(properties['correlation_id'])

	def publish(self, exchange, routing_key, message, properties=None, mandatory=False):
		"""
		Publish a message to an AMQP exchange.

		Parameters
		----------
		exchange: string
			Exchange to publish to.
		routing_key: string
			Routing key to use for this message.
		message: string
			Message to publish.
		properties: dict
			Properties to set on message. This parameter is optional, but if set, at least the following options must be set:
				content_type: string - what content_type to specify, default is 'text/plain'.
				delivery_mode: int - what delivery_mode to use. By default message are not persistent, but this can be
					set by specifying PERSISTENT_MESSAGE .
		mandatory: boolean
			If set to True, the mandatory bit will be set on the published message.
		"""
		return publish_message(self.channel, exchange, routing_key, message, properties, mandatory)

	def reply(self, original_headers, message, properties=None):
		"""
		Reply to a RPC request. This function will use the default exchange, to directly contact the reply_to queue.

		Parameters
		----------
		original_headers: dict
			The headers of the originating message that caused this reply.
		message: string
			Message to reply with
		properties: dict
			Properties to set on message. This parameter is optional, but if set, at least the following options must be set:
				content_type: string - what content_type to specify, default is 'text/plain'.
				delivery_mode: int - what delivery_mode to use. By default message are not persistent, but this can be
					set by specifying PERSISTENT_MESSAGE .
		"""
		rpc_reply(self.channel, original_headers, message, properties)


def rpc_reply(channel, original_headers, message, properties=None):
	"""
	Reply to a RPC request. This function will use the default exchange, to directly contact the reply_to queue.

	Parameters
	----------
	channel: object
		Properly initialized AMQP channel to use.
	original_headers: dict
		The headers of the originating message that caused this reply.
	message: string
		Message to reply with
	properties: dict
		Properties to set on message. This parameter is optional, but if set, at least the following options must be set:
			content_type: string - what content_type to specify, default is 'text/plain'.
			delivery_mode: int - what delivery_mode to use. By default message are not persistent, but this can be
				set by specifying PERSISTENT_MESSAGE .
	"""
	if not properties:
		properties = {}
	properties['correlation_id'] = original_headers.correlation_id

	publish_message(channel, '', original_headers.reply_to, message, properties)



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see 
# <http://www.gnu.org/licenses/>.

""" Helper functions for working with python multiprocessing. """

from __future__ import absolute_import
import multiprocessing, logging, os

class Workers(object):
	"""
	Container to register and handle multiple Worker processes.
	"""
	worker_list = {}

	def __init__(self):
		self.logger = logging.getLogger(__name__)

	def registerWorker(self, name, worker):
		"""
		Register a new Worker, under the given descriptive name.

		Trying to register multiple workers under the same name will raise an Exception.

		Parameters
		----------
		name: string
			Name to register the given worker under.
		worker: multiprocessing.Process, or a subclass
			Process object to register.
		"""
		if not isinstance(worker, multiprocessing.Process):
			self.logger.error("Process {0} is not actually a Process!".format(name))
			raise Exception("Process {0} is not actually a Process!".format(name))

		if name in self.worker_list:
			self.logger.error("Process {0} already registered!".format(name))
			raise Exception("Process {0} already registered!".format(name))

		self.worker_list[name] = worker
		self.logger.debug("Registered worker {0}".format(name))

		return worker

	def getWorkers(self):
		"""
		Retrieve a list of names of all registered Workers.
		"""
		return self.worker_list.keys()

	def getWorker(self, name):
		"""
		Retrieve the Worker registered under the given name.

		If the given name does not exists in the Worker list, an Exception is raised.

		Parameters
		----------
		name: string
			Name of the Worker to retrieve
		"""
		if not name in self.worker_list:
			self.logger.error("Worker {0} is not registered!".format(name))
			raise Exception("Worker {0} is not registered!".format(name))

		return self.worker_list[name]

	def unregisterWorker(self, name):
		"""
		Unregister the Worker registered under the given name.

		Make sure that the given Worker is properly stopped, or that a reference is
		kept in another place. Once unregistered, this class will not keep any
		other references to the Worker.

		Parameters
		----------
		name: string
			Name of the Worker to unregister
		"""
		if not name in self.worker_list:
			self.logger.error("Worker {0} is not registered!".format(name))
			raise Exception("Worker {0} is not registered!".format(name))

		del self.worker_list[name]
		self.logger.debug("Unregistered worker {0}".format(name))

	def startAll(self):
		"""
		Start all registered Workers.
		"""
		self.logger.info("Starting all workers...")

		for worker in self.getWorkers():
			process = self.getWorker(worker)
			self.logger.debug("Starting {0}".format(process.name))
			process.start()

		self.logger.info("Started all workers")

	def stopAll(self, timeout=10, stop=False):
		"""
		Stop all registered Workers. This is method assumes that the Worker has already
		received a stop message somehow, and simply joins the Process until it dies, as 
		follows:

		1. The Worker is retrieved.
		2. The Worker is joined, and will wait until the Worker exits.
		3. The Worker is unregistered.
		4. If $stop = True, the main process is killed.
		"""
		self.logger.info("Stopping all workers...")

		for worker in self.getWorkers():
			process = self.getWorker(worker)
			self.logger.debug("Stopping {0}".format(process.name))
			if process.is_alive():
				process.join(timeout)
				if process.is_alive():
					self.logger.warning("Failed to stop {0}, terminating".format(process.name))
					process.terminate()
			self.unregisterWorker(worker)

		self.logger.info("Stopped all workers")

		if stop:
			self.logger.fatal("Comitting suicide")
			os._exit(0)



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see
# <http://www.gnu.org/licenses/>.

""" AMQP consumer related classes and functions. """

import logging
import pika
from pika.exceptions import ChannelClosed

class Queue(object):
	""" Holds a connection to an AMQP queue, and methods to consume from it. """
	def __init__(self, host, credentials, queue, binds=None, prefetch_count=4):
		"""
		Initialize AMQP connection.

		Parameters
		----------
		host: tuple
			Must contain hostname and port to use for connection
		credentials: tuple
			Must contain username and password for this connection
		queue: dict
			Must contain at least the following keys:
				queue: string - what queue to use
				passive: boolean - should we use an existing queue, to try to declare our own
			Options below are optional when passive = True
				durable: boolean - should the queue be durable
				auto_delete: boolean - should we auto delete the queue when we close the connection
		binds: list of dicts
			A list of dicts with the following keys:
				queue: string - name of the queue to bind
				exchange: string - name of the exchange to bind
				routing_key: string - routing key to use for this bind
		prefetch_count: int
			Define how many items may be prefetched at a time.
			The default value of 4 is a workaround of an issue that exists in python-pika 0.9.13. See
			the Github issue for more info: https://github.com/pika/pika/issues/286 .
		"""
		self.logger = logging.getLogger(__name__)

		self.logger.info("Creating AMQP connection to {0}:{1}".format(host[0], host[1]))
		self.credentials = pika.PlainCredentials(credentials[0], credentials[1])
		self.parameters = pika.ConnectionParameters(host=host[0], port=host[1], credentials=self.credentials)
		self.connection = pika.BlockingConnection(self.parameters)
		self.channel = self.connection.channel()
		self.channel.basic_qos(prefetch_count=4)

		self.queue_name = queue['queue']
		self.logger.info("Declaring queue {0}".format(self.queue_name))
		self.channel.queue_declare(**queue)

		if binds:
			self._perform_binds(binds)

	def _perform_binds(self, binds):
		"""
		Binds queues to exchanges.

		Parameters
		----------
		binds: list of dicts
			a list of dicts with the following keys:
				queue: string - name of the queue to bind
				exchange: string - name of the exchange to bind
				routing_key: string - routing key to use for this bind
		"""
		for bind in binds:
			self.logger.debug("Binding queue {0} to exchange {1} with key {2}".format(bind['queue'], bind['exchange'], bind['routing_key']))
			self.channel.queue_bind(**bind)

	def _perform_unbinds(self, binds):
		"""
		Unbinds queues from exchanges.

		Parameters
		----------
		binds: list of dicts
			A list of dicts with the following keys:
				queue: string - name of the queue to bind
				exchange: string - name of the exchange to bind
				routing_key: string - routing key to use for this bind
		"""
		for bind in binds:
			self.logger.debug("Unbinding queue {0} from exchange {1} with key {2}".format(bind['queue'], bind['exchange'], bind['routing_key']))
			self.channel.queue_unbind(**bind)

	def close(self):
		"""
		Closes the internal connection.
		"""
		self.cancel()
		self.logger.debug("Closing AMQP connection")
		try:
			self.connection.close()
		except Exception, eee:
			self.logger.warning("Received an error while trying to close AMQP connection: " + str(eee))

	def cancel(self, consumer_tag=None):
		"""
		Cancels the current consuming action by using the stored consumer_tag. If a consumer_tag is given, that one is used instead.

		Parameters
		----------
		consumer_tag: string
			Tag of consumer to cancel
		"""
		if not consumer_tag:
			if not hasattr(self, "consumer_tag"):
				return
			consumer_tag = self.consumer_tag
		self.channel.basic_cancel(consumer_tag)

	def consume(self, consumer_callback, exclusive=False, recover=False):
		"""
		Initialize consuming of messages from an AMQP queue. Messages will be consumed after start_consuming() is called.

		Calling this method multiple times with exclusive active will result in a pika.exceptions.ChannelClosed Error.

		Parameters
		----------
		consumer_callback: callback
			Function to call when a message is consumed. The callback function will be called on each delivery,
			and will receive three parameters:
				* channel
				* method_frame
				* header_frame
				* body
		exclusive: boolean
			Is this consumer supposed to be the exclusive consumer of the given queue?
		recover: boolean
			Asks the server to requeue all previously delivered but not acknowledged messages. This can be used to recover from a sudden
			disconnect or other error.

		Returns
		-------
		string
			Returns a generated consumer_tag.
		"""
		if recover:
			self.logger.info("Asking server to requeue all unacknowledged messages")
			self.channel.basic_recover(requeue=True)
		self.consumer_tag = self.channel.basic_consume(consumer_callback=consumer_callback, queue=self.queue_name, exclusive=exclusive)
		return self.consumer_tag

	def start_consuming(self):
		"""
		Start consuming messages.
		"""
		self.channel.start_consuming()

	def stop_consuming(self):
		"""
		Stop consuming messages.
		"""
		self.channel.stop_consuming()



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see
# <http://www.gnu.org/licenses/>.

import shlex, os
from subprocess import Popen, PIPE, STDOUT

def call_system(command):
	"""
	Calls a command using os.system. This method only returns the
	return code of the process. If you want process output, use
	call simple_cli instead.
	"""

	return os.system(command)

def call_simple_cli(command, cwd=None, universal_newlines=False, redirect_stderr=False):
	""" Simple wrapper around SimpleCliTool. Simple. """
	return SimpleCliTool()._call_cli(command, cwd, universal_newlines, redirect_stderr)

class SimpleCliTool(object):
	envvars = {}

	def add_env(self, var, value):
		"""
		Store a custom environment value internally. This value is used on every
		call to _call_cli.
		"""
		self.envvars[var] = value

	def del_env(self, var):
		"""
		Delete a custom environment value. This will raise an exception if the
		variable does not exist.
		"""
		del(self.envvars[var])

	def get_env(self):
		"""
		Return the internally stored dict of environment variables.
		"""
		return self.envvars

	def _call_cli(self, command, cwd=None, universal_newlines=False, redirect_stderr=False):
		"""
		Executes the given command, internally using Popen. The output of
		stdout and stderr are returned as a tuple. The returned tuple looks
		like: (stdout, stderr, returncode)

		Parameters
		----------
		command: string
			The command to execute.
		cwd: string
			Change the working directory of the program to the specified path.
		universal_newlines: boolean
			Enable the universal_newlines feature of Popen.
		redirect_stderr: boolean
			If True, redirect stderr into stdout
		"""
		command = str(command.encode("utf-8").decode("ascii", "ignore"))
		env = os.environ.copy()
		env.update(self.envvars)
		stderr = STDOUT if redirect_stderr else PIPE
		proc = Popen(shlex.split(command), stdout=PIPE, stderr=stderr, cwd=cwd, universal_newlines=universal_newlines, env=env)
		stdout, stderr = proc.communicate()

		return (stdout, stderr, proc.returncode)



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see
# <http://www.gnu.org/licenses/>.

"""
Helper functions for working with the Python built-in logging module.
"""

from __future__ import absolute_import
import collections
import logging
import logging.handlers
import os


def get_logger(name=None, level=logging.NOTSET, handlers=None):
	"""
	Create a Python logging Logger for the given name. A special case is
	when the name is None, as this will represent the root Logger object.

	When handlers are specified, the currently configured handlers for this name
	are removed, and the specified handlers are set.

	Parameters
	----------
	name: string
		Name of the Logger to create. Specify None to designate the root Logger.
	level: string
		One of: CRITICAL, ERROR, WARNING, INFO or DEBUG. Alternatively, use the `logging`
		constants: logging.CRITICAL, logging.ERROR, etc.
	handlers: dict
		Keys specifies the handler, value may optionally contain configuration,
		or be specified as None.

		Supported handlers are:
		- console: logging to stdout. Optionally specify a custom Handler using 'handler'.
		- file: logging to a specific file. Specify the file as 'logfile'.
		- syslog: logging to syslog.

		All handlers support custom output formats by specifying a 'format'.
	"""
	logger = logging.getLogger(name)

	if name is None:
		name = "root"
	if handlers is None:
		handlers = []

	logger.setLevel(level)

	if len(handlers) != 0:
		logger.handlers = []

	if "console" in handlers:
		if not isinstance(handlers['console'], collections.Iterable):
			handlers['console'] = {}

		if "handler" in handlers['console']:
			strm = handlers['console']['handler']
		else:
			strm = logging.StreamHandler()

		if "format" in handlers['console']:
			fmt = logging.Formatter(handlers['console']['format'])
		else:
			fmt = logging.Formatter('%(message)s')

		strm.setLevel(level)
		strm.setFormatter(fmt)
		logger.addHandler(strm)

	if "file" in handlers:
		if not isinstance(handlers['file'], collections.Iterable):
			raise TypeError("file handler config must be a dict")
		if "logfile" not in handlers['file']:
			raise ValueError("file handler config must contain logfile path name")

		fil = logging.handlers.WatchedFileHandler(handlers['file']['logfile'])

		if "format" in handlers['file']:
			fmt = logging.Formatter(handlers['file']['format'])
		else:
			fmt = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

		fil.setLevel(level)
		fil.setFormatter(fmt)
		logger.addHandler(fil)

	if "syslog" in handlers:
		if not isinstance(handlers['syslog'], collections.Iterable):
			handlers['syslog'] = {}

		sysl = logging.handlers.SysLogHandler(address='/dev/log', facility=logging.handlers.SysLogHandler.LOG_SYSLOG)

		if "format" in handlers['syslog']:
			fmt = logging.Formatter(handlers['syslog']['format'])
		else:
			fmt = logging.Formatter('%(name)s[%(process)s] %(levelname)-8s: %(message)s')

		sysl.setLevel(level)
		sysl.setFormatter(fmt)
		logger.addHandler(sysl)

	return logger



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see 
# <http://www.gnu.org/licenses/>.

import gdbm, json, collections

def dump_simple_db(path):
	"""
	Dumps a SimpleDb as string in the following format:
	<key>: <json-encoded string>
	"""
	output = []
	simpledb = SimpleDb(path, mode="r", sync=False)
	with simpledb as db:
		for key in db:
			output.append("{0}: {1}".format(key, db.dumpvalue(key)))
	return "\n".join(output)

class SimpleDb(collections.MutableMapping):
	"""
	Implements a simple key/value store based on GDBM. Values are stored as JSON strings,
	to allow for more complex values than GDBM itself can provide.

	This class implements the full MutableMapping ABC, which means that after using open(),
	or using with, this class behaves as a dict. All changes will be saved to disk after
	using close() or ending the with statement.
	"""

	def __init__(self, path, mode = "c", sync=True):
		"""
		Store the given parameters internally and prepare for opening the database later.

		Arguments
		---------
		path: string
			Path where to create or open the database
		mode: string
			What mode to use when opening the database:
			- "r" (read-only)
			- "w" (read-write)
			- "c" (read-write, create when not exists)
			- "n" (read-write, always create new file)
		sync: boolean
			If set to True, data will be flushed to disk after every change.
		"""
		self.path = path
		self.mode = mode + ("s" if sync else "")
		self.db = None

	def _checkopen(self):
		if self.db == None:
			raise RuntimeError("SimpleDb was not opened")

	def open(self):
		"""
		Open the GDBM database internally.
		"""
		return self.__enter__()

	def close(self):
		"""
		Close the internal GDBM database.
		"""
		self.__exit__(None, None, None)

	def dumpvalue(self, key):
		"""
		Retrieves the given key, and returns the raw JSON encoded string.
		"""
		self._checkopen()
		return self.db[key]

	def __enter__(self):
		self.db = gdbm.open(self.path, self.mode)
		return self

	def __exit__(self, exc_type, exc_value, traceback):
		self._checkopen()
		self.db.sync()
		self.db.close()
		self.db = None

	def __getitem__(self, key):
		self._checkopen()
		return json.loads(self.db[key])

	def __setitem__(self, key, value):
		self._checkopen()
		self.db[key] = json.dumps(value)

	def __delitem__(self, key):
		self._checkopen()
		del(self.db[key])

	def __iter__(self):
		self._checkopen()
		key = self.db.firstkey()

		while key != None:
			yield key
			key = self.db.nextkey(key)

		if key == None:
			raise StopIteration()

	def __len__(self):
		self._checkopen()
		return len(self.db)



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see
# <http://www.gnu.org/licenses/>.

""" AMQP exchange related classes and functions. """

import logging
import pika

NORMAL_MESSAGE = 1
PERSISTENT_MESSAGE = 2


class Exchange(object):
	""" Holds a connection to an AMQP exchange, and methods to publish to it. """
	def __init__(self, host, credentials, exchange=None, routing_key=None):
		"""
		Initialize AMQP connection.

		Parameters
		----------
		host: tuple
			Must contain hostname and port to use for connection
		credentials: tuple
			Must contain username and password for this connection
		exchange: dict
			Must contain at least the following keys:
				exchange: string - what exchange to use
				exchange_type: string - what type of exchange to use ("direct")
				passive: boolean - should we use an existing exchange, or try to declare our own
			Options below are optional when passive = True
				durable: boolean - should the exchange be durable
				auto_delete: boolean - should we auto delete the exchange when we close the connection
		routing_key: string
			what routing_key to use for published messages. If unset, this parameter must be set during publishing
		"""
		self.logger = logging.getLogger(__name__)

		self.logger.debug("Creating connection to {0}:{1}".format(host[0], host[1]))
		self.default_routing_key = routing_key
		self.credentials = pika.PlainCredentials(credentials[0], credentials[1])
		self.parameters = pika.ConnectionParameters(host=host[0], port=host[1], credentials=self.credentials)
		self.connection = pika.BlockingConnection(self.parameters)
		self.channel = self.connection.channel()

		if exchange:
			self.exchange_name = exchange['exchange']
			self.logger.debug("Declaring exchange {0}".format(self.exchange_name))
			self.channel.exchange_declare(**exchange)
		else:
			self.exchange_name = None

	def close(self):
		"""
		Closes the internal connection.
		"""
		self.logger.debug("Closing AMQP connection")
		self.connection.close()

	def publish(self, message, properties=None, mandatory=False):
		"""
		Publish a message to an AMQP exchange.

		Parameters
		----------
		message: string
			Message to publish.
		properties: dict
			Properties to set on message. This parameter is optional, but if set, at least the following options must be set:
				content_type: string - what content_type to specify, default is 'text/plain'.
				delivery_mode: int - what delivery_mode to use. By default message are not persistent, but this can be
					set by specifying PERSISTENT_MESSAGE .
			The following options are also available:
				routing_key: string - what routing_key to use. MUST be set if this was not set during __init__.
				exchange: string - what exchange to use. MUST be set if this was not set during __init__.
		mandatory: boolean
			If set to True, the mandatory bit will be set on the published message.

		Returns
		-------
		Depending on the mode of the Channel, the return value can signify different things:

		basic_Confirm is active:
			True means that the message has been delivered to a queue, False means it hasn't.
		mandatory bit was set on message:
			True means that the message has been delivered to a consumer, False means that it has been returned.
		No special bit or mode has been set:
			None is returned.
		"""
		return publish_message(self.channel, self.exchange_name, self.default_routing_key, message, properties, mandatory)


def publish_message(channel, exchange, routing_key, message, properties=None, mandatory=False):
	"""
	Publish a message to an AMQP exchange.

	Parameters
	----------
	channel: object
		Properly initialized AMQP channel to use.
	exchange: string
		Exchange to publish to.
	routing_key: string
		Routing key to use for this message.
	message: string
		Message to publish.
	properties: dict
		Properties to set on message. This parameter is optional, but if set, at least the following options must be set:
			content_type: string - what content_type to specify, default is 'text/plain'.
			delivery_mode: int - what delivery_mode to use. By default message are not persistent, but this can be
				set by specifying PERSISTENT_MESSAGE .
		The following options are also available:
			routing_key: string - what routing_key to use. Will override the one set in the parameters.
			exchange: string - what exchange to use. Will override the one set in the parameters.
	mandatory: boolean
		If set to True, the mandatory bit will be set on the published message.

	Returns
	-------
	Depending on the mode of the Channel, the return value can signify different things:

	basic_Confirm is active:
		True means that the message has been delivered to a queue, False means it hasn't.
	mandatory bit was set on message:
		True means that the message has been delivered to a consumer, False means that it has been returned.
	No special bit or mode has been set:
		None is returned.
	"""
	if properties is None:
		properties = {}
	if properties and "routing_key" in properties:
		routing_key = properties["routing_key"]
		del(properties["routing_key"])
	if properties and "exchange" in properties:
		exchange = properties["exchange"]
		del(properties["exchange"])

	if not routing_key:
		raise ValueError("routing_key was not specified")
	if not exchange and not exchange == "":
		raise ValueError("exchange was not specified")

	logging.getLogger(__name__ + ".publish_message").debug("Publishing message to exchange {0} with routing_key {1}".format(exchange, routing_key))

	return channel.basic_publish(exchange, routing_key, message, pika.BasicProperties(**properties), mandatory)



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see 
# <http://www.gnu.org/licenses/>.

""" Helper functions for file and directory globbing. """

from __future__ import absolute_import
import os, fnmatch

class Globber(object):
	"""
	Traverses a directory and returns all absolute filenames.
	"""

	def __init__(self, path, include=None, recursive=True):
		"""
		Initialize Globber parameters. Filter may be a list of globbing patterns.

		Parameters
		----------
		path: string
			Absolute path to the directory to glob
		include: list of strings
			List of globbing pattern strings. By default, ALL files in the given path
			are globbed.
		recursive: boolean
			When True: will traverse subdirectories found in $path. Defaults to True.
		"""
		if include is None:
			include = ['*']
		self.path = path
		self.include = include
		self.recursive = recursive
	
	def glob(self):
		"""
		Traverse directory, and return all absolute filenames of files that
		match the globbing patterns.
		"""
		matches = []
		for root, dirnames, filenames in os.walk(self.path):
			if not self.recursive:
				while len(dirnames) > 0:
					dirnames.pop()
			for include in self.include:
				for filename in fnmatch.filter(filenames, include):
					matches.append(os.path.join(root, filename))
		return matches



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see 
# <http://www.gnu.org/licenses/>.

""" Helper functions for working with python multithreading. """

from __future__ import absolute_import
import threading, logging, os

class Threads(object):
	"""
	Container to register and handle multiple Threads.
	"""
	thread_list = {}

	def __init__(self):
		self.logger = logging.getLogger(__name__)

	def registerThread(self, name, thread):
		"""
		Register a new Thread , under the given descriptive name.

		Trying to register multiple threads under the same name will raise an Exception.

		Parameters
		----------
		name: string
			Name to register the given thread under.
		thread: threading.Thread, or a subclass
			Thread object to register.
		"""
		if not isinstance(thread, threading.Thread):
			self.logger.error("Thread {0} is not actually a Thread!".format(name))
			raise Exception("Thread {0} is not actually a Thread!".format(name))

		if name in self.thread_list:
			self.logger.error("Thread {0} already registered!".format(name))
			raise Exception("Thread {0} already registered!".format(name))

		self.thread_list[name] = thread
		self.logger.debug("Registered thread {0}".format(name))

		return thread

	def getThreads(self):
		"""
		Retrieve a list of names of all registered Threads.
		"""
		return self.thread_list.keys()

	def getThread(self, name):
		"""
		Retrieve the Thread registered under the given name.

		If the given name does not exists in the Thread list, an Exception is raised.

		Parameters
		----------
		name: string
			Name of the Thread to retrieve
		"""
		if not name in self.thread_list:
			self.logger.error("Thread {0} is not registered!".format(name))
			raise Exception("Thread {0} is not registered!".format(name))

		return self.thread_list[name]

	def unregisterThread(self, name):
		"""
		Unregister the Thread registered under the given name.

		Make sure that the given Thread is properly stopped, or that a reference is
		kept in another place. Once unregistered, this class will not keep any
		other references to the Thread.

		Parameters
		----------
		name: string
			Name of the Thread to unregister
		"""
		if not name in self.thread_list:
			self.logger.error("Thread {0} is not registered!".format(name))
			raise Exception("Thread {0} is not registered!".format(name))

		del self.thread_list[name]
		self.logger.debug("Unregistered thread {0}".format(name))

	def startAll(self):
		"""
		Start all registered Threads.
		"""
		self.logger.info("Starting all threads...")

		for thread in self.getThreads():
			thr = self.getThread(thread)
			self.logger.debug("Starting {0}".format(thr.name))
			thr.start()

		self.logger.info("Started all threads")

	def stopAll(self, stop=False):
		"""
		Stop all registered Threads. This is method assumes that the Thread is using
		and internal variable called stop to control its main loop. Stopping a Thread
		is achieved as follows:

		1. The Thread is retrieved.
		2. $thread.stop is set to False.
		3. The Thread is joined, and will wait until the Thread exits.
		4. The Thread is unregistered.
		5. If $exit = True, the main process is killed.

		Ensure that any registered Thread responds to having its stop property set to False,
		else calling stopAll() will result in a hung process.
		"""
		self.logger.info("Stopping all threads...")

		for thread in self.getThreads():
			thr = self.getThread(thread)
			self.logger.debug("Stopping {0}".format(thr.name))
			thr.stop = True
			thr.join()
			self.unregisterThread(thread)

		self.logger.info("Stopped all threads")

		if stop:
			self.logger.fatal("Comitting suicide")
			os._exit(0)



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see 
# <http://www.gnu.org/licenses/>.

""" Helper functions for making Threads execute periodically. """

from __future__ import absolute_import
import threading, datetime, time, logging

class Scheduler(threading.Thread):
	"""
	A single thread that is automatically called with a specific interval.

	To stop a thread that is in its main loop, set stop to True.
	"""
	def __init__(self, delay, action, name, startNow=False, *args, **kwargs):
		"""
		Initialize a new Scheduler thread.

		Parameters
		----------
		delay: int
			The delay between consequtive runs of this thread, in seconds.
		action: function pointer
			The function to call.
		name: string
			Descriptive name of this thread.
		startNow: boolean
			When True, this thread will start immediately when run() is called.
			When False, this thread will start now+interval seconds when run() is called.
		*args
			Positional arguments to pass to action.
		**kwargs:
			Keyword arguments to pass to action.
		"""

		self.logger = logging.getLogger(name)
		super(Scheduler, self).__init__(None, None, name, None, None)

		self.delay = delay
		self.main_action = action
		self.name = name
		self.main_args = args
		self.main_kwargs = kwargs

		self.stop = False
		now = datetime.datetime.now()
		if startNow is True:
			self.lastRun = datetime.datetime.min
			self.logger.debug("Thread {0} will start immediately".format(name))
		else:
			self.lastRun = now
			if isinstance(startNow, (int, long)):
				self.lastRun += datetime.timedelta(seconds=startNow)
			wait = (self.lastRun - now).seconds + delay
			self.logger.debug("Thread {0} will start in {1} seconds".format(name, wait))
	
	def setStartAction(self, action, *args, **kwargs):
		"""
		Set a function to call when run() is called, before the main action is called.

		Parameters
		----------
		action: function pointer
			The function to call.
		*args
			Positional arguments to pass to action.
		**kwargs:
			Keyword arguments to pass to action.
		"""

		self.init_action = action
		self.init_args = args
		self.init_kwargs = kwargs
	
	def setStopAction(self, action, *args, **kwargs):
		"""
		Set a function to call when run() is stopping, after the main action is called.

		Parameters
		----------
		action: function pointer
			The function to call.
		*args
			Positional arguments to pass to action.
		**kwargs:
			Keyword arguments to pass to action.
		"""
		self.stop_action = action
		self.stop_args = args
		self.stop_kwargs = kwargs

	def run(self):
		"""
		Calls the defined action every $interval seconds. Optionally calls an action before
		the main loop, and an action when stopping, if these are defined.

		Exceptions in the main loop will NOT cause the thread to die.
		"""
		self.logger.debug("Thread {0} is entering main loop".format(self.name))

		if hasattr(self, "init_action"):
			self.logger.debug("Thread {0} is calling its init action")
			self.init_action(*self.init_args, **self.init_kwargs)

		while not self.stop:
			self.logger.debug("Delay is {0}".format(self.delay))
			if (datetime.datetime.now() - self.lastRun).total_seconds() > self.delay:
				self.logger.debug("Thread {0} is running".format(self.name))
				try:
					self.main_action(*self.main_args, **self.main_kwargs)
				except Exception:
					self.logger.exception("Thread {0} generated an exception!".format(self.name))

				self.lastRun = datetime.datetime.now()
				self.logger.debug("Thread {0} is done".format(self.name))
			time.sleep(1)

		if hasattr(self, "stop_action"):
			self.logger.debug("Thread {0} is calling its stop action")
			self.stop_action(*self.stop_args, **self.stop_kwargs)

		self.logger.debug("Thread {0} is exiting main loop".format(self.name))



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see 
# <http://www.gnu.org/licenses/>.

"""
Helper functions for loading ConfigObj configuration files.

Care must be taken to catch any ParseErrors that the underlying ConfigObj code may raise,
these functions explicitely do not catch them.
"""

from __future__ import absolute_import
import logging, os, inspect
from configobj import ConfigObj, ConfigObjError
from validate import Validator
from .globber import Globber

def get_config(config_base, custom_file=None, configspec=None):
	"""
	Loads a configuration file from multiple locations, and merge the results into one.

	This function will load configuration files from a number of locations in sequence,
	and will overwrite values in the previous level if they are redefined in the current.

	The levels are in sequence:

	1. Distribution level configuration in the program directory called $config_base.config.
	2. System-wide level configuration in /etc/$config_base.config
	3. User level configuration in ~/.$config_base.config
	4. An optionally specified $custom_file

	Parameters
	----------
	config_base: string
		Basename of the configuration file, typically the same as the name of the program.
	custom_file: string
		Absolute path to a custom configuration file.
	configspec: ConfigObj
		Used to sanitize the values in the resulting ConfigObj. Validation errors are currently
		not exposed to the caller.
	"""

	logger = logging.getLogger(__name__)

	logger.debug("Expanding variables")
	home = os.path.expanduser("~")
	loc = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))

	logger.debug("Create empty config")
	config = ConfigObj()

	# Merge in config file in program dir
	if os.path.isfile(os.path.join(loc, "%s.config" % config_base)):
		logger.debug("Loading config from workingdir")
		cfg = ConfigObj(os.path.join(loc, "%s.config" % config_base), configspec=configspec)
		if configspec:
			cfg.validate(Validator())
		config.merge(cfg)

	# Merge in system-wide config (Unix specific)
	if os.path.isfile("/etc/%s.config" % config_base):
		logger.debug("Loading config from /etc")
		cfg = ConfigObj("/etc/%s.config" % config_base, configspec=configspec)
		if configspec:
			cfg.validate(Validator())
		config.merge(cfg)

	# Merge in user specific config
	if os.path.isfile(os.path.join(home, ".%s.config" % config_base)):
		logger.debug("Loading config from homedir")
		cfg = ConfigObj(os.path.join(home, ".%s.config" % config_base), configspec=configspec)
		if configspec:
			cfg.validate(Validator())
		config.merge(cfg)

	# Config file provided on command line has preference
	if custom_file:
		logger.debug("Loading custom config file")
		cfg = ConfigObj(custom_file, configspec=configspec)
		if configspec:
			cfg.validate(Validator())
		config.merge(cfg)

	return config

def get_config_dir(path, pattern="*.config", configspec=None, allow_errors=False):
	"""
	Load an entire directory of configuration files, merging them into one.

	This function will load multiple configuration files matching the given pattern,
	in the given path, and merge them. The found files are first sorted alphabetically,
	and then loaded and merged. A good practice is to use ConfigObj sections, for easy
	loading of information like per-host configuration.

	Parameters
	----------
	path: string
		Absolute path to a directory of ConfigObj files
	pattern: string
		Globbing pattern used to find files. Defaults to *.config.
	configspec: ConfigObj
		Used to sanitize the values in the resulting ConfigObj. Validation errors are currently
		not exposed to the caller.
	allow_errors: boolean
		If False, errors raised by ConfigObj are not caught.
		If True, errors raise by ConfigObj are caught, and an error is logged using logger.
	"""

	logger = logging.getLogger(__name__)

	logger.debug("Loading all files matching {0} in {1}".format(pattern, path))
	files = Globber(path, include=[pattern], recursive=False).glob()
	files = sorted(files)

	config = ConfigObj()

	for filename in files:
		logger.debug("- Loading config for {0}".format(filename))
		try:
			conf = ConfigObj(filename, configspec=configspec)
		except ConfigObjError, coe:
			logger.error("An error occurred while parsing {0}: {1}".format(filename, str(coe)))
			continue
		if configspec:
			conf.validate(Validator())
		config.merge(conf)

	return config



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see
# <http://www.gnu.org/licenses/>.


class MessageNotDelivered(IOError):
    """
    Exception to be raised when an AMQP message could not be queued.
    """
    pass


class MessageDeliveryTimeout(IOError):
    """
    Exception to be raised when an AMQP message could not be delivered in the specified period.
    """
    pass



#!/usr/bin/env python
# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see
# <http://www.gnu.org/licenses/>.

from os.path import join, dirname
from setuptools import setup
from chaos.version import NAME, VERSION

DESCRIPTION = "chaos Python modules: Assorted libraries and subroutines"
LONG_DESCRIPTION = "A set of libraries and subroutines I commonly use in my python projects."

setup(
	name=NAME,
	version=VERSION,
	description=DESCRIPTION,
	long_description=LONG_DESCRIPTION,
	license="GNU LGPLv3+",
	author="Nick Douma",
	author_email="n.douma@nekoconeko.nl",
	url="https://github.com/LordGaav/python-chaos",
	platforms=["any"],
	classifiers=[
		"Development Status :: 4 - Beta",
		"Intended Audience :: Developers",
		"License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
		"Operating System :: Unix",
		"Programming Language :: Python :: 2 :: Only",
		"Programming Language :: Python :: 2.7",
		"Topic :: Software Development :: Libraries :: Python Modules"
	],
	packages=[
		"chaos",
		"chaos.amqp",
		"chaos.multiprocessing",
		"chaos.threading"
	],
	install_requires=[
		"configobj",
		"pika>=0.9.5"
	]
)



# Copyright (c) 2014 Nick Douma < n.douma [at] nekoconeko . nl >
#
# This file is part of chaos, a.k.a. python-chaos .
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see 
# <http://www.gnu.org/licenses/>.

"""
Helper functions for loading command line arguments using argparse.
"""

from __future__ import absolute_import
import logging
from argparse import ArgumentParser

def get_config_argparse(suppress=None):
	"""
	Create an ArgumentParser which listens for the following common options:
	- --config
	- --help
	- --quiet
	- --verbose
	- --version

	Arguments
	---------
	suppress_help: list of strings
		Defines what options to suppress from being added to the ArgumentParser. Provide
		the name of the options to suppress, without "--". Of note in particular is the help option.

		If help is suppressed, help information is suppressed in the returned ArgumentParser
		If help is not suppressed, help information is automatically output by ArgumentParser 
			when -h/--help is passed, and the program is exited upon parsing the arguments.
		In both cases, -h/--help is not actually suppressed, only the behaviour of the ArgumentParser
		when encountering the option.
	"""

	if suppress is None:
		suppress = []

	config_parser = ArgumentParser(description="Looking for config", add_help=(not "help" in suppress))
	if not "config" in suppress:
		config_parser.add_argument("--config",	metavar="CFG",		 type=str,		help="Config file to load")
	if "help" in suppress:
		config_parser.add_argument("--help",	action="store_true", default=False, help="Display usage information and exit")
	if not "quiet" in suppress:
		config_parser.add_argument("--quiet",	action="store_true", default=False, help="Don't print messages to stdout")
	if not "verbose" in suppress:
		config_parser.add_argument("--verbose", action="store_true", default=False, help="Output debug messages")
	if not "version" in suppress:
		config_parser.add_argument("--version", action="store_true", default=False, help="Display version information and exit")

	return config_parser

def get_config_arguments():
	"""
	Parse command line arguments, and try to find common options. Internally
	this method uses the ArgumentParser returned by get_config_argparse().

	All variables are stored as True if set, --config will contain a string.

	Returns a tuple containing parsed variables and unknown variables,
	just like ArgumentParser.parse_known_args() would.
	"""

	logger = logging.getLogger(__name__)
	logger.debug("Parsing configuration arguments")

	return get_config_argparse(suppress=["help"]).parse_known_args()

def get_default_config_file(argparser, suppress=None, default_override=None):
	"""
	Turn an ArgumentParser into a ConfigObj compatible configuration file.

	This method will take the given argparser, and loop over all options contained. The configuration file is formatted
	as follows:

	# <option help info>
	<option destination variable>=<option default value>

	Arguments
	---------
	argparser: ArgumentParser
	suppress: list of strings
		All options specified will be suppressed from the config file. Useful to avoid adding stuff like version or help.
	default_override: dict
		This method will use the defaults from the given ArgumentParser, unless the option is specified here. If specified,
		the default from this dict will be used instead. The format is { "option": <new default value>, ... } .
	"""
	if not suppress:
		suppress = []
	if not default_override:
		default_override = {}

	lines = []
	seen_arguments = []
	for arg in argparser._actions:
		if arg.dest in suppress:
			continue
		if arg.dest in seen_arguments:
			continue
		default = arg.default
		if arg.dest in default_override.keys():
			default = default_override[arg.dest]
		lines.append("# {0}\n{1}={2}\n".format(arg.help, arg.dest, default))
		seen_arguments.append(arg.dest)

	return "".join(lines)
